from django.db import models
from .settings import User
from django.utils.translation import ugettext_lazy as _


class PermissionTypes(models.TextChoices):
    common = "common", u"Page and Api"
    api = "api", u"Api"
    page = "page", u"Page"


class Role(models.Model):
    name = models.CharField(max_length=64, unique=True)
    comment = models.TextField(_("Comment"), null=True, blank=True)
    user = models.ManyToManyField(User, blank=True, through="UserRoleMembership",
                                  verbose_name=u"Related User-Role")
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = verbose_name
        ordering = ('-created',)


class UserRoleMembership(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    role = models.ForeignKey(Role, on_delete=models.CASCADE)

    def __str__(self):
        return '{} has {}'.format(self.user.username, self.role.name)

    class Meta:
        verbose_name = _("User Role")
        verbose_name_plural = verbose_name
        unique_together = (
            ('user', 'role'),
        )


class HttpMethod(models.Model):
    name = models.CharField(max_length=64, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Http Methods")
        verbose_name_plural = verbose_name


class Api(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    alias_name = models.CharField(_("Alias Name"), max_length=64, null=True, blank=True)
    url = models.CharField(_("URL"), unique=True, max_length=1024)
    comment = models.TextField(_("Comment"), null=True, blank=True)
    method = models.ManyToManyField(HttpMethod, through='ApiMethodMembership')
    username = models.CharField(_("Operation User"), max_length=64, null=True, blank=True)

    def __str__(self):
        return "{}({})".format(self.url, self.alias_name)

    class Meta:
        ordering = ('-created',)
        verbose_name = _("Api")
        verbose_name_plural = verbose_name


class ApiMethodMembership(models.Model):
    api = models.ForeignKey(Api, on_delete=models.CASCADE)
    method = models.ForeignKey(HttpMethod, on_delete=models.CASCADE)

    def __str__(self):
        return '{}-{}'.format(self.api.url, self.method.name)

    class Meta:
        verbose_name = _("api method")
        verbose_name_plural = verbose_name
        unique_together = (
            ('api', 'method'),
        )


class Permission(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    name = models.CharField(_("Permission Name"), max_length=64, unique=True)
    alias_name = models.CharField(_("Alias Name"), max_length=128, null=True, blank=True)
    type = models.CharField(_("Permission Type"), max_length=64, db_index=True,
                            default=PermissionTypes.common, choices=PermissionTypes.choices)
    role = models.ManyToManyField(Role, related_name='role_permission')
    status = models.BooleanField(_("effective"), default=True)
    api = models.ManyToManyField(ApiMethodMembership, blank=True, verbose_name=u"Related Api-Method")
    username = models.CharField(_("Operation User"), max_length=128, null=True, blank=True)
    comment = models.TextField(_("Comment"), null=True, blank=True)

    def __str__(self):
        return '{}({})'.format(self.name, self.alias_name)

    class Meta:
        ordering = ('-created',)
        verbose_name = _("Role Permission")
        verbose_name_plural = verbose_name
        unique_together = (
            ('name', 'type',),
        )


class UserToPermission(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    user = models.OneToOneField(User, on_delete=models.CASCADE,
                                related_name='user_permission')
    permission = models.ManyToManyField(Permission)

    class Meta:
        ordering = ('-created',)
        verbose_name = _("User Permission")
        verbose_name_plural = verbose_name



# -*- coding: UTF-8 -*-
import os
import pickle
import sys
from datetime import datetime

from django.core.management import BaseCommand
from django.db import transaction
from rest_framework import serializers
from django.db.models import Q
from drf_rbac.settings import User, PERMISSION_PATH, PERMISSION_RELATIVE_PATH
from drf_rbac.models import HttpMethod, Role, Api, ApiMethodMembership, Permission, UserToPermission


def progress_bar(finish_tasks_number, tasks_number):
    percentage = round(finish_tasks_number / tasks_number * 100) if tasks_number else 100
    print(f"\rrate: {percentage}%", "█" * (percentage // 2), end="")
    sys.stdout.flush()


class HttpMethodSerializer(serializers.ModelSerializer):

    class Meta:
        model = HttpMethod
        exclude = ["id"]


class RoleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Role
        exclude = ["id", "created"]


class SimpleRoleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Role
        fields = ["name"]


class ApiSerializer(serializers.ModelSerializer):

    class Meta:
        model = Api
        exclude = ["id", "method", "created"]


class ApiMethodMembershipSerializer(serializers.ModelSerializer):
    api__url = serializers.SerializerMethodField()
    method__name = serializers.SerializerMethodField()

    def get_api__url(self, obj):
        return obj.api.url

    def get_method__name(self, obj):
        return obj.method.name

    class Meta:
        model = ApiMethodMembership
        fields = ("api__url", "method__name")


class PermissionSerializer(serializers.ModelSerializer):
    api = serializers.SerializerMethodField()
    role = serializers.SerializerMethodField()

    def get_api(self, obj):
        return ApiMethodMembershipSerializer(obj.api.all(), many=True).data

    def get_role(self, obj):
        return SimpleRoleSerializer(obj.role.all(), many=True).data

    class Meta:
        model = Permission
        exclude = ["id", "created"]


class SimplePermissionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Permission
        fields = ["name", "type"]


class UserSimpleSerializer(serializers.ModelSerializer):

    class Meta:
        model = User
        fields = ('username',)


class UserToPermissionSerializer(serializers.ModelSerializer):
    user = serializers.SerializerMethodField()
    permission = serializers.SerializerMethodField()

    def get_user(self, obj):
        return obj.user.username

    def get_permission(self, obj):
        return SimplePermissionSerializer(obj.permission.all(), many=True).data

    class Meta:
        model = UserToPermission
        exclude = ["id", "created"]


class Command(BaseCommand):
    """
        import or export permission
    """
    def export_model(self, file_path, *args):
        for table_name in args:
            model, model_serializer = globals()[table_name], globals()[f"{table_name}Serializer"]
            self.write({"table": table_name,
                        "value": model_serializer(model.objects.all(), many=True).data}, file_path)

    def import_common_model(self, qd):
        if qd.get("table") == "Role" and qd.get("table") in self.__class__.STEP_POOL:
            print(f"step 1.1：import {qd.get('table')} ...")
            for i, j in enumerate(qd.get("value")):
                progress_bar(i, len(qd.get("value")) - 1)
                self.model_create(Role, j, Q(name=j.get("name")), "Role")
        elif qd.get("table") == "HttpMethod" and qd.get("table") in self.__class__.STEP_POOL:
            print(f"\nstep 1.2：import {qd.get('table')} ...")
            for i, j in enumerate(qd.get("value")):
                progress_bar(i, len(qd.get("value")) - 1)
                self.model_create(HttpMethod, j, Q(name=j.get("name")), "HttpMethod")

    def import_api_model(self, qd):
        if qd.get("table") == "Api" and qd.get("table") in self.__class__.STEP_POOL:
            print(f"\nstep 2：import {qd.get('table')} ...")
            for i, j in enumerate(qd.get("value")):
                progress_bar(i, len(qd.get("value")) - 1)
                q_obj = Q(url=j.get("url"))
                self.model_create(Api, j, q_obj, "Api")

    def import_apimethodmembership_model(self, qd):
        if qd.get("table") == "ApiMethodMembership" and qd.get("table") in self.__class__.STEP_POOL:
            print(f"\nstep 3：import {qd.get('table')} ...")
            for i, j in enumerate(qd.get("value")):
                progress_bar(i, len(qd.get("value")) - 1)
                q_obj = Q(api__url=j.get("api__url")) & \
                        Q(method__name=j.get("method__name"))
                self.model_create(ApiMethodMembership, j, q_obj, "ApiMethodMembership")

    def _update_api_ship_for_permission(self, apis, permission_obj):
        for api in apis:
            api_qs = Api.objects.filter(url=api.get("api__url"))
            method_qs = HttpMethod.objects.filter(name=api.get("method__name"))
            if api_qs.exists() and method_qs.exists():
                api_ship_qs = ApiMethodMembership.objects.filter(api=api_qs.first(),
                                                                 method=method_qs.first())
                api_ship_qs.exists() and permission_obj.api.add(api_ship_qs.first())

    def _update_role_for_permission(self, roles, permission_obj):
        for role in roles:
            role_qs = Role.objects.filter(name=role.get("name"))
            role_qs.exists() and permission_obj.role.add(role_qs.first())

    def _update_permission_for_usertopermission(self, permissions, user_to_permission_obj):
        for perm in permissions:
            perm_qs = Permission.objects.filter(name=perm.get("name"), type=perm.get("type"))
            perm_qs.exists() and user_to_permission_obj.permission.add(perm_qs.first())

    def import_permission_model(self, qd):
        if qd.get("table") == "Permission" and qd.get("table") in self.__class__.STEP_POOL:
            print(f"\nstep 4：import {qd.get('table')} ...")
            for i, j in enumerate(qd.get("value")):
                progress_bar(i, len(qd.get("value")) - 1)
                q_obj = Q(name=j.get("name"))
                au_qs = Permission.objects.filter(q_obj)
                apis, roles = j.get("api"), j.get("role")
                if not au_qs.exists():
                    mutating_data = self._data_mutating("Permission", j)
                    permission_obj = Permission.objects.create(**mutating_data)
                else:
                    permission_obj = au_qs.first()
                self._update_api_ship_for_permission(apis, permission_obj)
                self._update_role_for_permission(roles, permission_obj)

    def import_usertopermission_model(self, qd):
        if qd.get("table") == "UserToPermission" and qd.get("table") in self.__class__.STEP_POOL:
            print(f"\nstep 5：import {qd.get('table')} ...")
            for i, j in enumerate(qd.get("value")):
                progress_bar(i, len(qd.get("value")) - 1)
                username = j.get("user")
                user_to_permission_qs = UserToPermission.objects.filter(user__username=username)
                permissions = j.get("permission")
                if not user_to_permission_qs.exists():
                    mutating_data = self._data_mutating("UserToPermission", j)
                    if not isinstance(mutating_data.get("user"), str):
                        user_to_permission_obj = UserToPermission.objects.create(**mutating_data)
                        self._update_permission_for_usertopermission(permissions, user_to_permission_obj)
                else:
                    user_to_permission_obj = user_to_permission_qs.first()
                    self._update_permission_for_usertopermission(permissions, user_to_permission_obj)

    def model_create(self, model, instance_data, q_obj, table):
        if not model.objects.filter(q_obj).exists():
            mutating_data = self._data_mutating(table, instance_data)
            model.objects.create(**mutating_data)

    def _data_mutating(self, table, instance_data):
        if table == "ApiMethodMembership":
            api_qs = Api.objects.filter(url=instance_data.get("api__url"))
            method_qs = HttpMethod.objects.filter(name=instance_data.get("method__name"))
            if api_qs.exists() and method_qs.exists():
                del instance_data["api__url"]
                del instance_data["method__name"]
                instance_data["api"] = api_qs.first()
                instance_data["method"] = method_qs.first()
        elif table == "Permission":
            del instance_data["api"]
            del instance_data["role"]
        elif table == "UserToPermission":
            user_qs = User.objects.filter(username=instance_data.get("user"))
            if user_qs.exists():
                instance_data["user"] = user_qs.first()
            del instance_data["permission"]
        return instance_data

    def write(self, data, file_path):
        with open(file_path, "ab+") as f:
            pickle.dump(data, f)

    def search_permission_file(self, path, file):
        for parent_dir, _, file_names in os.walk(path):
            for walk_file_name in file_names:
                if walk_file_name.startswith(file):
                    file_path = os.path.join(parent_dir, walk_file_name)
                    return file_path
            else:
                raise ValueError("file not exists")

    def assemble_import_file(self, file_path):
        if not file_path:
            raise ValueError("file path not exists")

        elif "/" in file_path:
            if not file_path.startswith("/"):
                raise ValueError("offer abs path")

            if file_path.endswith("/"):
                raise ValueError("offer file name")

            dir_name, file_name = file_path.rsplit("/", 1)

            if not os.path.exists(dir_name):
                raise ValueError("abs path not exists")

            file_path = self.search_permission_file(dir_name, file_name)
        else:
            file_path = self.search_permission_file(os.path.join(PERMISSION_PATH, PERMISSION_RELATIVE_PATH), file_path)
        return file_path

    def read_all(self, file_path=None):
        res = list()
        with open(file_path, "rb") as f:
            while True:
                try:
                    res.append(pickle.load(f))
                except EOFError:
                    break
        return res

    def _check_dir_path(self, path_name):
        commands_dir = os.path.join(PERMISSION_PATH, path_name)
        not os.path.exists(commands_dir) and os.makedirs(commands_dir)
        return commands_dir

    def format_export_file_name(self):
        p_path = self._check_dir_path(PERMISSION_RELATIVE_PATH)
        file_path = os.path.join(p_path, f"permissions-{datetime.now().strftime('%H%M%m%d%Y')}.pickle")
        if os.path.exists(file_path):
            os.remove(file_path)
        return file_path

    def step_trigger(self, options):
        self.__class__.STEP_POOL = list()
        step_map = {"1": ["HttpMethod", "Role"],
                    "2": ["Api"],
                    "3": ["ApiMethodMembership"],
                    "4": ["Permission"],
                    "5": ["UserToPermission"]}
        step = options.get("step")
        if step:
            self.__class__.STEP_POOL.extend(step_map.get(step))
        else:
            for value in step_map.values():
                self.__class__.STEP_POOL.extend(value)

    def add_arguments(self, parser):
        parser.add_argument('--action', help='import/export/test')
        parser.add_argument('--path', help='import必填项: 直接输入文件名则在 pandora-be/data/permission_files 中寻找，'
                                           '找不到抛出错误，输入绝对路径校验成功后导入')
        parser.add_argument('--step', help='1/2/3/4/5')

    @transaction.atomic
    def handle(self, *args, **options):
        start_time = datetime.now()
        print("loading...")
        if options.get("action") == "test":
            print(f"Api\n{ApiSerializer(Api.objects.all()[:2], many=True).data}\n")
            print(f"HttpMethod\n{HttpMethodSerializer(HttpMethod.objects.all(), many=True).data}\n")
            print(f"Role\n{RoleSerializer(Role.objects.all(), many=True).data}\n")
            print(f"ApiMethodMembership\n{ApiMethodMembershipSerializer(ApiMethodMembership.objects.all()[:2], many=True).data}\n")
            print(f"Permission\n{PermissionSerializer(Permission.objects.filter(api__isnull=False, role__isnull=False)[:5], many=True).data}\n")
            print(f"UserToPermission\n{UserToPermissionSerializer(UserToPermission.objects.all(), many=True).data}\n")
        elif options.get("action") == "export":
            file_path = self.format_export_file_name()
            self.export_model(file_path, "Role", "HttpMethod")
            self.export_model(file_path, "Api")
            self.export_model(file_path, "ApiMethodMembership")
            self.export_model(file_path, "Permission")
            self.export_model(file_path, "UserToPermission")
            print(f"action: export\nfile: {file_path}\ntime: {(datetime.now() - start_time).seconds}s")
        elif options.get("action") == "import":
            self.step_trigger(options)
            file_path = self.assemble_import_file(options.get("path"))
            for qd in self.read_all(file_path):
                self.import_common_model(qd)  # step 1
                self.import_api_model(qd)  # step 2
                self.import_apimethodmembership_model(qd)  # step 3
                self.import_permission_model(qd)  # step 4
                self.import_usertopermission_model(qd)  # step 5
            print(f"\naction: import\ntime: {(datetime.now() - start_time).seconds}s\nsource: {file_path}")



# Generated by Django 3.0.5 on 2022-11-18 08:35

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('drf_rbac', '0002_auto_20221118_0826'),
    ]

    operations = [
        migrations.AlterField(
            model_name='role',
            name='user',
            field=models.ManyToManyField(blank=True, through='drf_rbac.UserRoleMembership', to=settings.AUTH_USER_MODEL, verbose_name='Related User-Role'),
        ),
    ]



import re
from rest_framework import permissions
from .models import UserRoleMembership, UserToPermission
from django.db import connection
from .settings import PERMISSION_START_WHITE_LIST, PERMISSION_END_WHITE_LIST, SUPER_ROLE_LIST


ROLE_AUTH_SQL = "SELECT papi.id FROM drf_rbac_permission_role par " \
      "INNER JOIN drf_rbac_permission_api pap ON pap.permission_id = par.permission_id " \
      "INNER JOIN drf_rbac_apimethodmembership pa ON pap.apimethodmembership_id = pa.id " \
      "INNER JOIN drf_rbac_api papi ON papi.id = pa.api_id " \
      "INNER JOIN drf_rbac_httpmethod pt ON pt.id = pa.method_id " \
      "WHERE par.role_id IN ({}) AND pt.name = '{}' AND (papi.url = '{}' or papi.url = '{}')"

OBJ_AUTH_SQL = "SELECT papi.id FROM drf_rbac_usertopermission_permission pua " \
       "INNER JOIN drf_rbac_usertopermission pu ON pua.usertopermission_id = pu.id " \
       "INNER JOIN drf_rbac_permission_api pap ON pap.permission_id = pua.permission_id " \
       "INNER JOIN drf_rbac_apimethodmembership pa ON pap.apimethodmembership_id = pa.id " \
       "INNER JOIN drf_rbac_api papi ON papi.id = pa.api_id " \
       "INNER JOIN drf_rbac_httpmethod pt ON pt.id = pa.method_id " \
       "WHERE pu.user_id = {} AND pt.name = '{}' AND (papi.url = '{}' or papi.url = '{}')"


class UserRolePermission(permissions.BasePermission):
    """
    Global permission check for User.
    """

    def has_permission(self, request, view):
        try:
            if request.u:
                new_path = request.path
                user_role_qs = UserRoleMembership.objects.filter(user=request.user)

                # SUPER ROLE
                if user_role_qs.filter(role__name__in=SUPER_ROLE_LIST):
                    return True

                # No role
                role_id_list = [str(user_role.role.id) for user_role in user_role_qs]
                if not role_id_list:
                    return False

                # start white
                for white_list_api in PERMISSION_START_WHITE_LIST:
                    if re.match(r"{}{}".format(white_list_api, r".*"), new_path):
                        return True

                # end white
                for white_list_api in PERMISSION_END_WHITE_LIST:
                    if re.match(r"{}{}$".format(r".*", white_list_api), new_path):
                        return True

                # delete and detail
                # DELETE,GET,PUT,PATCH match /api/paas/cluster/234/ --> /api/paas/cluster/
                if re.match(r'(.*)/\d+/$',  new_path):
                    new_path = re.sub(r'/\d+', '', new_path)

                # /api/paas/cluster/2/turn/ --> /api/paas/cluster/{}/turn/
                if re.match(r'(.*)/\d+/\w+/$',  new_path):
                    new_path = re.sub(r'/\d+/', '/{}/', new_path)

                # role perm and user perm
                role_id_format = ",".join(list(set(role_id_list)))
                new_path_format_end = new_path.rsplit("/", 1)[0]

                cursor = connection.cursor()
                cursor.execute(ROLE_AUTH_SQL.format(role_id_format, request.method.lower(),
                                                    new_path, new_path_format_end))
                result = cursor.fetchall()

                def obj_perm(cr):
                    if UserToPermission.objects.filter(user=request.u).exists():
                        cr.execute(OBJ_AUTH_SQL.format(request.u.id, request.method.lower(),
                                                       new_path, new_path_format_end))
                        return cr.fetchall()

                if result or obj_perm(cursor):
                    return True
        except Exception as e:
            raise ValueError(e)



from __future__ import unicode_literals
from django.contrib import admin
from django.db import transaction
from .models import Role, Api, HttpMethod, ApiMethodMembership, Permission, UserToPermission, UserRoleMembership
from django import forms
from django.utils.html import format_html


def role_validate(value):
    if value not in [role.name for role in Role.objects.all()]:
        raise forms.ValidationError(u'name error')


class RoleForm(forms.ModelForm):
    clone = forms.BooleanField(required=False, widget=forms.NullBooleanSelect, initial=False)
    clone_role_field = forms.CharField(required=False,
                                       widget=forms.TextInput(attrs={'placeholder': u'role name'}),
                                       validators=[role_validate])


class UsersInline(admin.TabularInline):
    model = UserRoleMembership


class RoleAdmin(admin.ModelAdmin):
    form = RoleForm
    list_display = ('name', 'show_permission_num',)
    search_fields = ('name', 'comment')
    inlines = [
        UsersInline,
    ]

    def show_permission_num(self, obj):
        count = obj.role_permission.count()
        text = "<a href='/drf_rbac/role_perm/{}/'>{}</a>".format(obj.id, count)
        return format_html(text)

    show_permission_num.short_description = "permission count"

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if not change:
            clone_role = form.cleaned_data.get("clone_role_field")
            if form.cleaned_data.get("clone") and clone_role:
                for i in Permission.objects.filter(role=Role.objects.filter(name=clone_role).first()):
                    i.role.add(obj)


class HttpMethodAdmin(admin.ModelAdmin):
    list_display = ('name',)
    search_fields = ('name',)


class MethodInline(admin.TabularInline):
    model = ApiMethodMembership


class ApiAdmin(admin.ModelAdmin):
    list_display = ('url', 'alias_name', 'show_method', 'show_permission', 'username', 'updated', 'created',)
    list_editable = ('alias_name',)
    search_fields = ('id', 'alias_name', 'url', 'username',)
    list_display_links = ('url',)
    filter_horizontal = ('method',)
    list_filter = ('username',)
    actions = ['copy']
    list_per_page = 10
    ordering = ('-pk',)
    fieldsets = [
        (None, {'fields': ('url', 'alias_name',)}),
    ]
    inlines = [
        MethodInline,
    ]

    @transaction.atomic
    def copy(self, request, queryset):
        for instance in queryset:

            obj = Api.objects.create(alias_name="{}-copy?".format(instance.alias_name),
                                     url="?{}".format(instance.url),
                                     username=instance.username)
            ship_list = []
            for method in instance.method.all():
                ship_list.append(ApiMethodMembership.objects.create(api=obj, method=method))

            for ship_obj in ApiMethodMembership.objects.filter(api=instance):
                for perm in ship_obj.permission_set.all():
                    perm.api.add(*ship_list)

        self.message_user(request, "copy completed!")

    copy.short_description = "copy"

    def show_permission(self, obj):
        apimethodmembership = obj.apimethodmembership_set.all()
        results = []
        for mem in apimethodmembership:
            for permission in mem.permission_set.all():
                results.append("{},{}".format(permission.id, permission.name))
        res = ""
        for i in set(results):
            _, name = i.split(",")
            res += "<a href='/admin/drf_rbac/permission/?q={name}'>{name}</a> </br>".format(name=name)
        text = """<div>
                    <div style="margin-left:20px;margin-bottom:5px;margin-top:5px">
                    <a style="border-style:outset;padding:2px" onclick="
                         var parent= window.event.target.parentNode;
                         if(parent.nextElementSibling){{
                            console.log(parent.nextElementSibling.style.display)
                            if(parent.nextElementSibling.style.display=='block'){{
                                parent.nextElementSibling.style.display='none';
                            }}else{{
                                parent.nextElementSibling.style.display='block';
                            }}
                        }}else {{
                            parent.nextElementSibling.style.display='none';
                        }}">click</a></div>
                    <div style="display:none">""" + res + "</div></div>"
        return format_html(text)

    show_permission.short_description = "permission"

    def show_method(self, obj):
        method = obj.method.all()
        results = []
        for m in method:
            results.append(m.name)
        return " ".join(results)

    show_method.short_description = "http methods"

    def save_model(self, request, obj, form, change):
        obj.username = request.user.username
        super(__class__, self).save_model(request, obj, form, change)


class PermissionAdmin(admin.ModelAdmin):
    list_display = ('name', 'alias_name', 'type', 'status', 'username', 'updated', 'created')
    search_fields = ('name', 'alias_name', 'comment')
    list_editable = ('alias_name', 'status')
    ordering = ('-pk',)
    date_hierarchy = 'created'
    # empty_value_display = "---"
    list_display_links = ('name',)
    list_per_page = 20
    radio_fields = {"type": admin.HORIZONTAL}
    actions = ["copy"]
    fieldsets = [
        ('common info', {'fields': ('type', ('name', 'alias_name',), 'status', 'comment')}),
        ('role info', {'fields': ('role',)}),
        ('api info', {'fields': ('api',),
                      'classes': ['wide'],
                      'description': 'Attached API information'}),
    ]
    filter_horizontal = ('role', 'api',)
    list_filter = ('type', 'username')
    save_as = True
    save_on_top = True

    @transaction.atomic
    def copy(self, request, queryset):
        for instance in queryset:
            obj = Permission.objects.create(alias_name="{}-copy?".format(instance.alias_name),
                                            name="?{}".format(instance.name),
                                            type=instance.type,
                                            status=instance.status,
                                            username=instance.username,
                                            comment=instance.comment)

            obj.role.add(*instance.role.all())
            obj.api.add(*instance.api.all())

        self.message_user(request, "copy completed!")

    copy.short_description = "copy"

    def save_model(self, request, obj, form, change):
        obj.username = request.user.username
        super(__class__, self).save_model(request, obj, form, change)


class UserToPermissionAdmin(admin.ModelAdmin):
    list_display = ('user', 'created', 'updated')
    filter_horizontal = ('permission',)


admin.site.register(Role, RoleAdmin)
admin.site.register(HttpMethod, HttpMethodAdmin)
admin.site.register(Api, ApiAdmin)
admin.site.register(Permission, PermissionAdmin)
admin.site.register(UserToPermission, UserToPermissionAdmin)



from django.conf import settings

User = settings.AUTH_USER_MODEL

PERMISSION_PATH = settings.BASE_DIR
PERMISSION_RELATIVE_PATH = 'permission_files'

PERMISSION_START_WHITE_LIST = getattr(settings, "PERMISSION_START_WHITE_LIST", [])
PERMISSION_END_WHITE_LIST = getattr(settings, "PERMISSION_END_WHITE_LIST", [])
SUPER_ROLE_LIST = getattr(settings, "SUPER_ROLE_LIST", [])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup
from os import path as os_path
import os

this_directory = os_path.abspath(os_path.dirname(__file__))
URL = 'https://github.com/MangoodLuck/drf-rbac-plus.git'
NAME = 'drf-rbac-plus'
VERSION = '1.3.0'
DESCRIPTION = 'rbac permission plus'
if os.path.exists('README.md'):
    with open('README.md', encoding='utf-8') as f:
        LONG_DESCRIPTION = f.read()
else:
    LONG_DESCRIPTION = DESCRIPTION
AUTHOR = 'Xiao Man'
AUTHOR_EMAIL = 'cloudbye@163.com'
LICENSE = 'MIT'
PLATFORMS = [
    'linux',
]

def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description

def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]

setup(
    name=NAME,
    version=VERSION,
    description=(
        DESCRIPTION
    ),
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=AUTHOR,
    maintainer_email=AUTHOR_EMAIL,
    license=LICENSE,
    packages=find_packages(),
    platforms=PLATFORMS,
    url=URL,
    install_requires=read_requirements('requirements.txt'),
    python_requires=">=3.5",
)



# Generated by Django 3.0.5 on 2022-11-18 03:32

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Api',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('alias_name', models.CharField(blank=True, max_length=64, null=True, verbose_name='Alias Name')),
                ('url', models.CharField(max_length=1024, unique=True, verbose_name='URL')),
                ('comment', models.TextField(blank=True, null=True, verbose_name='Comment')),
                ('username', models.CharField(blank=True, max_length=64, null=True, verbose_name='Operation User')),
            ],
            options={
                'verbose_name': 'Api',
                'verbose_name_plural': 'Api',
                'ordering': ('-created',),
            },
        ),
        migrations.CreateModel(
            name='ApiMethodMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_rbac.Api')),
            ],
            options={
                'verbose_name': 'api method',
                'verbose_name_plural': 'api method',
            },
        ),
        migrations.CreateModel(
            name='HttpMethod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
            ],
            options={
                'verbose_name': 'Http Methods',
                'verbose_name_plural': 'Http Methods',
            },
        ),
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=64, unique=True, verbose_name='Permission Name')),
                ('alias_name', models.CharField(blank=True, max_length=128, null=True, verbose_name='Alias Name')),
                ('type', models.CharField(choices=[('common', 'Page and Api'), ('api', 'Api'), ('page', 'Page')], db_index=True, default='common', max_length=64, verbose_name='Permission Type')),
                ('status', models.BooleanField(default=True, verbose_name='effective')),
                ('username', models.CharField(blank=True, max_length=128, null=True, verbose_name='Operation User')),
                ('comment', models.TextField(blank=True, null=True, verbose_name='Comment')),
                ('api', models.ManyToManyField(blank=True, to='drf_rbac.ApiMethodMembership', verbose_name='Related Api-Method')),
            ],
            options={
                'verbose_name': 'Role Permission',
                'verbose_name_plural': 'Role Permission',
                'ordering': ('-created',),
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
                ('comment', models.TextField(blank=True, null=True, verbose_name='Comment')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Role',
                'verbose_name_plural': 'Role',
                'ordering': ('-created',),
            },
        ),
        migrations.CreateModel(
            name='UserToPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('permission', models.ManyToManyField(to='drf_rbac.Permission')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='user_permission', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User Permission',
                'verbose_name_plural': 'User Permission',
                'ordering': ('-created',),
            },
        ),
        migrations.AddField(
            model_name='permission',
            name='role',
            field=models.ManyToManyField(related_name='role_permission', to='drf_rbac.Role'),
        ),
        migrations.AddField(
            model_name='apimethodmembership',
            name='method',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_rbac.HttpMethod'),
        ),
        migrations.AddField(
            model_name='api',
            name='method',
            field=models.ManyToManyField(through='drf_rbac.ApiMethodMembership', to='drf_rbac.HttpMethod'),
        ),
        migrations.AlterUniqueTogether(
            name='permission',
            unique_together={('name', 'type')},
        ),
        migrations.AlterUniqueTogether(
            name='apimethodmembership',
            unique_together={('api', 'method')},
        ),
    ]



from django.shortcuts import render
from .models import Role


def detail_auth(request, role_id):
    role = Role.objects.get(pk=role_id)
    auth = role.role_permission.all()
    res = []
    header = ['permission name', 'permission alias', 'permission type']
    for i in set(auth):
        name = "<a style='display:block' href='/admin/drf_rbac/permission/?q={name}'>{name}&nbsp</a>".format(
            name=i.name)
        res.append([name, i.alias_name, i.get_type_display()])
    return render(request, 'detail_auth.html', {"permission": res, "role_name": role.name, "header": header})



# Generated by Django 3.0.5 on 2022-11-18 08:26

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('drf_rbac', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserRoleMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_rbac.Role')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User Role',
                'verbose_name_plural': 'User Role',
                'unique_together': {('user', 'role')},
            },
        ),
        migrations.AddField(
            model_name='role',
            name='user',
            field=models.ManyToManyField(blank=True, null=True, through='drf_rbac.UserRoleMembership', to=settings.AUTH_USER_MODEL, verbose_name='Related User-Role'),
        ),
    ]
